/* Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */
package org.dive4elements.river.artifacts.sinfo.collision;

import java.util.ArrayList;
import java.util.Collection;

import org.dive4elements.river.artifacts.common.AbstractCalculationExportableResult;
import org.dive4elements.river.artifacts.common.ExportContextCSV;
import org.dive4elements.river.artifacts.common.ExportContextPDF;
import org.dive4elements.river.artifacts.common.GeneralResultType;
import org.dive4elements.river.artifacts.common.IExportContext;
import org.dive4elements.river.artifacts.common.MetaAndTableJRDataSource;
import org.dive4elements.river.artifacts.common.ResultRow;
import org.dive4elements.river.artifacts.sinfo.common.SInfoResultType;
import org.dive4elements.river.artifacts.sinfo.util.RiverInfo;

/**
 * Contains the results of a {@link FloodDurationCalculation}.
 *
 * @author Gernot Belger
 */
final class CollisionCalcDetailResult extends AbstractCalculationExportableResult {

    private static final long serialVersionUID = 1L;
    private static final String JASPER_FILE = "/jasper/templates/sinfo.collision.detail.jrxml";

    public CollisionCalcDetailResult(final String label, final Collection<ResultRow> rows) {
        super(label, rows);
    }

    @Override
    protected void writeCSVResultMetadata(final ExportContextCSV exportContextCSV) {

        // do nothing so far
    }

    @Override
    public void writeCSVHeader(final ExportContextCSV exportContextCSV, final RiverInfo river) {

        final int colSize = 6;
        exportContextCSV.writeTitleForTabs("sinfo.export.csv.title.collision.detail", 6); // Voraussetzung für Tabs ist, dass der Titel vor den Headern
        // geschrieben wird.
        // Das ist etwas doof.

        final Collection<String> header = new ArrayList<>(colSize);

        header.add(exportContextCSV.formatCsvHeader(GeneralResultType.station));
        header.add(exportContextCSV.formatCsvHeader(GeneralResultType.dateShort));
        header.add(exportContextCSV.msgUnitCSV(SInfoResultType.collisionGaugeW, SInfoResultType.collisionGaugeW.getUnit()));
        header.add(exportContextCSV.formatCsvHeader(GeneralResultType.gaugeLabelCm));
        header.add(exportContextCSV.msgUnitCSV(SInfoResultType.dischargeLong, SInfoResultType.dischargeLong.getUnit()));
        header.add(exportContextCSV.formatCsvHeader(SInfoResultType.dischargeZone));

        exportContextCSV.writeCSVLine(header.toArray(new String[colSize]));
    }

    @Override
    protected String[] formatCSVRow(final ExportContextCSV exportContextCSV, final ResultRow row) {
        return formatRow(exportContextCSV, row);
    }

    @Override
    protected String[] formatPDFRow(final ExportContextPDF exportContextCSV, final ResultRow row) {
        /*
         * final Collection<String> lines = new ArrayList<>(6);
         * lines.add(exportContextCSV.formatRowValue(row, BunduResultType.station));
         * return lines.toArray(new String[lines.size()]);
         */
        return formatRow(exportContextCSV, row);
    }

    private String[] formatRow(final IExportContext exportContext, final ResultRow row) {

        final Collection<String> lines = new ArrayList<>(6);

        lines.add(exportContext.formatRowValue(row, GeneralResultType.station));
        lines.add(exportContext.formatRowValue(row, GeneralResultType.dateShort));
        lines.add(exportContext.formatRowValue(row, SInfoResultType.collisionGaugeW));
        lines.add(exportContext.formatRowValue(row, GeneralResultType.gaugeLabelCm));
        lines.add(exportContext.formatRowValue(row, SInfoResultType.dischargeLong));
        lines.add(exportContext.formatRowValue(row, SInfoResultType.dischargeZone));

        return lines.toArray(new String[lines.size()]);
    }

    @Override
    protected String getJasperFile() {
        return JASPER_FILE;
    }

    @Override
    protected void addJRTableHeader(final ExportContextPDF exportContextCSV, final MetaAndTableJRDataSource source) {

        /* column headings */
        exportContextCSV.addJRMetadata(source, "station_header", GeneralResultType.station);
        exportContextCSV.addJRMetadata(source, "date_header", GeneralResultType.dateShort);
        exportContextCSV.addJRMetadata(source, "gaugew_header", SInfoResultType.collisionGaugeW);
        exportContextCSV.addJRMetadata(source, "gauge_header", GeneralResultType.gaugeLabelCm);
        exportContextCSV.addJRMetadata(source, "discharge_header", SInfoResultType.dischargeLong);
        exportContextCSV.addJRMetadata(source, "dischargezone_header", SInfoResultType.dischargeZone);
    }
}